/*
 * (C) 2007, Tom Wheeler.  All rights reserved.
 */
package com.tomwheeler.demo.jcr.worldinfo;

import java.util.*;
import javax.jcr.*;
import javax.jcr.query.Query;
import javax.jcr.query.QueryManager;
import javax.jcr.query.QueryResult;

/**
 * <p>
 * This class encapsulates repository interaction to provide simple
 * domain-specific data access methods.  Most of the JCR-specific
 * code in this application is inside this class.
 * </p>
 */
public class CountryManager {
    
    private Session session;
    
    public CountryManager(Session session) {
        this.session = session;
    }

    private static final String PROP_DATE_FORMED = "dateFormed";
    private static final String PROP_LANGUAGE = "language";
    private static final String PROP_CURRENCY = "currency";
    private static final String PROP_POPULATION = "population";
    private static final String PROP_TLD = "tld";

    public void add(Country country) throws DataAccessException {
        try {
            Node rootNode = session.getRootNode();

            Node continentNode = getOrCreate(rootNode, country.getContinent());
            if (continentNode.hasNode(country.getName())) {
                String msg = "Node already exists for '" + country.getName() + "'";
                throw new DataAccessException(msg);
            }

            Node countryNode = continentNode.addNode(country.getName());
            populateNode(countryNode, country);

            session.save();
        } catch (RepositoryException e) {
            throw new DataAccessException(e);
        }
    }
    
    public void update(Country country) throws DataAccessException {
        try {
            String path = "/" + country.getContinent() + "/" + country.getName();
            Node countryNode = (Node) session.getItem(path);

            // NOTE: I could avoid unnecessary some writes by checking which 
            // values actually differ between country object and its node.
            populateNode(countryNode, country);

            session.save();
        } catch (RepositoryException e) {
            throw new DataAccessException(e);
        }
    }

    public void delete(Country country) throws DataAccessException {
        try {
            String path = "/" + country.getContinent() + "/" + country.getName();

            Node node = (Node) session.getItem(path);

            if (node == null) {
                String msg = "Could not find '" + country.getName() + "' node to delete";
                throw new DataAccessException(msg);
            }

            node.remove();

            session.save();
        } catch (RepositoryException e) {
            throw new DataAccessException(e);
        }
    }

    public Country findByTld(String tld) throws DataAccessException {
        
        Country retval = null;
        try {
            Workspace ws = session.getWorkspace();            
            QueryManager qm = ws.getQueryManager();
            
            String qs = "//*[@tld='" + tld + "']";
            Query qry = qm.createQuery(qs, Query.XPATH);
            
            QueryResult res = qry.execute();
            NodeIterator resIter = res.getNodes();
            
            while (resIter.hasNext()) {
                Node countryNode = resIter.nextNode();
                retval = convertNodeToCountry(countryNode);
                break;
            }
            
        } catch (RepositoryException e) {
            throw new DataAccessException(e);
        }
        
        if (retval == null) {
            throw new DataAccessException("No country with TLD '" + tld + "'");
        }
        
        return retval;
    }

    public List<Country> getAllCountries() throws DataAccessException {
        try {
            List<Country> results = new ArrayList<Country>();

            Workspace ws = session.getWorkspace();
            QueryManager qm = ws.getQueryManager();

            // this query recursively finds any node with a currency property
            String qs = "//*[@currency]";
            Query qry = qm.createQuery(qs, Query.XPATH);

            QueryResult res = qry.execute();
            NodeIterator iter = res.getNodes();

            while (iter.hasNext()) {
                Node countryNode = iter.nextNode();
                Country country = convertNodeToCountry(countryNode);
                results.add(country);
            }
            
            return results;
        } catch (RepositoryException e) {
            throw new DataAccessException(e);
        }
    }

    /** helper method to return a node if it already exists, or create it if it does not */
    private static Node getOrCreate(Node parent, String nodeName) throws RepositoryException {

        Node retval = null;
        if (parent.hasNode(nodeName)) {
            retval = parent.getNode(nodeName);
        } else {
            retval = parent.addNode(nodeName);
        }

        return retval;
    }
    
    /** helper method to convert a country node's data to a country instance */
    private static Country convertNodeToCountry(Node countryNode) throws RepositoryException {
        String continent = countryNode.getParent().getName();

        String name = countryNode.getName();
        Calendar dateFormed = countryNode.getProperty(PROP_DATE_FORMED).getDate();
        String language = countryNode.getProperty(PROP_LANGUAGE).getString();
        String currency = countryNode.getProperty(PROP_CURRENCY).getString();
        long population = countryNode.getProperty(PROP_POPULATION).getLong();
        String tld = countryNode.getProperty(PROP_TLD).getString();

        Country country = new Country(name, continent);
        country.setCurrency(currency);
        country.setDateFormed(dateFormed);
        country.setPrincipalLanguage(language);
        country.setPopulation(population);
        country.setTld(tld);
        
        return country;
    }
 
    /** helper method to populate a country node based on the properties of a country instance */
    private static void populateNode(Node countryNode, Country country) throws RepositoryException {
        countryNode.setProperty(PROP_DATE_FORMED, country.getDateFormed());
        countryNode.setProperty(PROP_LANGUAGE, country.getPrincipalLanguage());
        countryNode.setProperty(PROP_CURRENCY, country.getCurrency());
        countryNode.setProperty(PROP_POPULATION, country.getPopulation());
        countryNode.setProperty(PROP_TLD, country.getTld());
    }
}